#include "fff.h"
#include "fff_blas.h"

#include <f2c.h>

#include <math.h>


/* TODO : add tests for dimension compatibility */ 

/* We have to account for the fact that BLAS assumes column-major
   ordered matrices by transposing */ 

#define FORTRAN_TRANS(TransA) ( (TransA)==(CblasNoTrans) ? "T" : "N" )
#define FORTRAN_UPLO(Uplo) ( (Uplo)==(CblasUpper) ? "L" : "U" )
#define FORTRAN_DIAG(Diag) ( (Diag)==(CblasUnit) ? "U" : "N" )
#define FORTRAN_SIDE(Side) ( (Side)==(CblasRight) ? "L" : "R" )

/* BLAS 1 */
extern doublereal ddot_(integer *n, doublereal *dx, integer *incx, doublereal *dy,
			integer *incy); 
extern doublereal dnrm2_(integer *n, doublereal *x, integer *incx); 
extern doublereal dasum_(integer *n, doublereal *dx, integer *incx); 
extern integer idamax_(integer *n, doublereal *dx, integer *incx); 
extern int dswap_(integer *n, doublereal *dx, integer *incx,
		  doublereal *dy, integer *incy); 
extern int dcopy_(integer *n, doublereal *dx, integer *incx,
		  doublereal *dy, integer *incy); 
extern int daxpy_(integer *n, doublereal *da, doublereal *dx,
		  integer *incx, doublereal *dy, integer *incy);
extern int dscal_(integer *n, doublereal *da, doublereal *dx,
		  integer *incx); 
extern int drotg_(doublereal *da, doublereal *db, doublereal *c__,
		  doublereal *s);
extern int drot_(integer *n, doublereal *dx, integer *incx,
		 doublereal *dy, integer *incy, doublereal *c__, doublereal *s); 
extern int drotmg_(doublereal *dd1, doublereal *dd2, doublereal *
		   dx1, doublereal *dy1, doublereal *dparam);
extern int drotm_(integer *n, doublereal *dx, integer *incx,
		  doublereal *dy, integer *incy, doublereal *dparam); 

/* BLAS 2 */
extern int dgemv_(char *trans, integer *m, integer *n, doublereal *
		  alpha, doublereal *a, integer *lda, doublereal *x, integer *incx,
		  doublereal *beta, doublereal *y, integer *incy);
extern int dtrmv_(char *uplo, char *trans, char *diag, integer *n,
		  doublereal *a, integer *lda, doublereal *x, integer *incx); 
extern int dtrsv_(char *uplo, char *trans, char *diag, integer *n,
		  doublereal *a, integer *lda, doublereal *x, integer *incx); 
extern int dsymv_(char *uplo, integer *n, doublereal *alpha,
		  doublereal *a, integer *lda, doublereal *x, integer *incx, doublereal
		  *beta, doublereal *y, integer *incy);
extern int dger_(integer *m, integer *n, doublereal *alpha,
		 doublereal *x, integer *incx, doublereal *y, integer *incy,
		 doublereal *a, integer *lda);
extern int dsyr_(char *uplo, integer *n, doublereal *alpha,
		 doublereal *x, integer *incx, doublereal *a, integer *lda); 
extern int dsyr2_(char *uplo, integer *n, doublereal *alpha,
		  doublereal *x, integer *incx, doublereal *y, integer *incy,
		  doublereal *a, integer *lda); 

/* BLAS 3 */ 
extern int dgemm_(char *transa, char *transb, integer *m, integer *
		  n, integer *k, doublereal *alpha, doublereal *a, integer *lda,
		  doublereal *b, integer *ldb, doublereal *beta, doublereal *c__,
		  integer *ldc); 
extern int dsymm_(char *side, char *uplo, integer *m, integer *n,
		  doublereal *alpha, doublereal *a, integer *lda, doublereal *b,
		  integer *ldb, doublereal *beta, doublereal *c__, integer *ldc); 
extern int dtrmm_(char *side, char *uplo, char *transa, char *diag,
		  integer *m, integer *n, doublereal *alpha, doublereal *a, integer *
		  lda, doublereal *b, integer *ldb); 
extern int dtrsm_(char *side, char *uplo, char *transa, char *diag,
		  integer *m, integer *n, doublereal *alpha, doublereal *a, integer *
		  lda, doublereal *b, integer *ldb); 
extern int dsyrk_(char *uplo, char *trans, integer *n, integer *k,
		  doublereal *alpha, doublereal *a, integer *lda, doublereal *beta,
		  doublereal *c__, integer *ldc); 
extern int dsyr2k_(char *uplo, char *trans, integer *n, integer *k,
		   doublereal *alpha, doublereal *a, integer *lda, doublereal *b,
		   integer *ldb, doublereal *beta, doublereal *c__, integer *ldc); 


/****** BLAS 1 ******/ 

/* Compute the scalar product x^T y for the vectors x and y, returning the result in result.*/
double fff_blas_ddot (const fff_vector * x, const fff_vector * y)
{
  int n = (int) x->size;
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 

  if ( n != y->size )
    return 1;  
 
  return( ddot_(&n, x->data, &incx, y->data, &incy) ); 
}

/* Compute the Euclidean norm ||x||_2 = \sqrt {\sum x_i^2} of the vector x. */ 
double fff_blas_dnrm2 (const fff_vector * x)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 

  return( dnrm2_(&n, x->data, &incx) ); 
}

/* Compute the absolute sum \sum |x_i| of the elements of the vector x.*/
double fff_blas_dasum (const fff_vector * x)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 

  return( dasum_(&n, x->data, &incx) ); 
}

/* 
   Return the index of the largest element of the vector x. The
   largest element is determined by its absolute magnitude. We
   substract one to the original Fortran routine an actual C index.
*/ 

CBLAS_INDEX_t fff_blas_idamax (const fff_vector * x)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 

  return( (CBLAS_INDEX_t)(idamax_(&n, x->data, &incx) - 1) ); 
}

/* Exchange the elements of the vectors x and y.*/
int fff_blas_dswap (fff_vector * x, fff_vector * y)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 
  
  if ( n != y->size )
    return 1;  
  
  return( dswap_(&n, x->data, &incx, y->data, &incy) ); 
}

/* Copy the elements of the vector x into the vector y */ 
int fff_blas_dcopy (const fff_vector * x, fff_vector * y)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 
  
  if ( n != y->size )
    return 1;  
  
  return( dcopy_(&n, x->data, &incx, y->data, &incy) ); 
}

/* Compute the sum y = \alpha x + y for the vectors x and y */ 
int fff_blas_daxpy (double alpha, const fff_vector * x, fff_vector * y)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 
  
  if ( n != y->size )
    return 1;  
  
  return( daxpy_(&n, &alpha, x->data, &incx, y->data, &incy) ); 
}

/* Rescale the vector x by the multiplicative factor alpha. */ 
int fff_blas_dscal (double alpha, fff_vector * x)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 
  
  return( dscal_(&n, &alpha, x->data, &incx) ); 
}


/* Compute a Givens rotation (c,s) which zeroes the vector (a,b),

          [  c  s ] [ a ] = [ r ]
          [ -s  c ] [ b ]   [ 0 ]

	  The variables a and b are overwritten by the routine. */
int fff_blas_drotg (double a[], double b[], double c[], double s[])
{
  return( drotg_(a, b, c, s) );
} 

/* Apply a Givens rotation (x', y') = (c x + s y, -s x + c y) to the vectors x, y.*/
int fff_blas_drot (fff_vector * x, fff_vector * y, double c, double s)
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 
  
  if ( n != y->size )
    return 1;  
  
  return( drot_(&n, x->data, &incx, y->data, &incy, &c, &s) ); 
}

/* Compute a modified Givens transformation. The modified Givens
   transformation is defined in the original Level-1 blas
   specification. */
int fff_blas_drotmg (double d1[], double d2[], double b1[], double b2, double P[])
{
  return( drotmg_(d1, d2, b1, &b2, P) ); 
}

    
/* Apply a modified Givens transformation.*/
int fff_blas_drotm (fff_vector * x, fff_vector * y, const double P[])
{
  int n = (int) x->size; 
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 
  
  if ( n != y->size )
    return 1;  
  
  return( drotm_(&n, x->data, &incx, y->data, &incy, (double*)P) ); 
}



/****** BLAS 2 ******/ 

/* Compute the matrix-vector product and sum y = \alpha op(A) x +
   \beta y, where op(A) = A, A^T, A^H for TransA = CblasNoTrans,
   CblasTrans, CblasConjTrans. */ 
int fff_blas_dgemv (CBLAS_TRANSPOSE_t TransA, double alpha, 
		    const fff_matrix * A, const fff_vector * x, double beta, fff_vector * y)
{
  char* trans = FORTRAN_TRANS(TransA); 
  int incx = (int) x->stride; 
  int incy = (int) y->stride;
  int m = (int) A->size2; 
  int n = (int) A->size1; 
  int lda = (int) A->tda; 

  return( dgemv_(trans, &m, &n, 
		 &alpha, 
		 A->data, &lda, 
		 x->data, &incx, 
		 &beta, 
		 y->data, &incy) ); 
}


/* Compute the matrix-vector product x = op(A) x for the triangular
   matrix A, where op(A) = A, A^T, A^H for TransA = CblasNoTrans,
   CblasTrans, CblasConjTrans. When Uplo is CblasUpper then the upper
   triangle of A is used, and when Uplo is CblasLower then the lower
   triangle of A is used. If Diag is CblasNonUnit then the diagonal of
   the matrix is used, but if Diag is CblasUnit then the diagonal
   elements of the matrix A are taken as unity and are not referenced.*/ 

int fff_blas_dtrmv (CBLAS_UPLO_t Uplo, CBLAS_TRANSPOSE_t TransA, CBLAS_DIAG_t Diag, 
		    const fff_matrix * A, fff_vector * x)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  char* trans = FORTRAN_TRANS(TransA); 
  char* diag = FORTRAN_DIAG(Diag); 
  int incx = (int) x->stride; 
  int n = (int) A->size1; 
  int lda = (int) A->tda; 

  return( dtrmv_(uplo, trans, diag, &n, 
		 A->data, &lda,
		 x->data, &incx) ); 

}

/* 
Compute inv(op(A)) x for x, where op(A) = A, A^T, A^H for TransA =
CblasNoTrans, CblasTrans, CblasConjTrans. When Uplo is CblasUpper then
the upper triangle of A is used, and when Uplo is CblasLower then the
lower triangle of A is used. If Diag is CblasNonUnit then the diagonal
of the matrix is used, but if Diag is CblasUnit then the diagonal
elements of the matrix A are taken as unity and are not referenced.
*/
int fff_blas_dtrsv (CBLAS_UPLO_t Uplo, CBLAS_TRANSPOSE_t TransA, CBLAS_DIAG_t Diag,
		    const fff_matrix * A, fff_vector * x)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  char* trans = FORTRAN_TRANS(TransA); 
  char* diag = FORTRAN_DIAG(Diag); 
  int incx = (int) x->stride; 
  int n = (int) A->size1; 
  int lda = (int) A->tda; 

  return( dtrsv_(uplo, trans, diag, &n, 
		 A->data, &lda, 
		 x->data, &incx) ); 
}

/* 
Compute the matrix-vector product and sum y = \alpha A x + \beta y for
the symmetric matrix A. Since the matrix A is symmetric only its upper
half or lower half need to be stored. When Uplo is CblasUpper then the
upper triangle and diagonal of A are used, and when Uplo is CblasLower
then the lower triangle and diagonal of A are used.
*/

int fff_blas_dsymv (CBLAS_UPLO_t Uplo, 
		    double alpha, const fff_matrix * A, 
		    const fff_vector * x, double beta, fff_vector * y)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  int incx = (int) x->stride; 
  int incy = (int) y->stride;
  int n = (int) A->size1; 
  int lda = (int) A->tda; 

  return( dsymv_(uplo, &n, 
		 &alpha, 
		 A->data, &lda, 
		 x->data, &incx, 
		 &beta, 
		 y->data, &incy) ); 
}

/* Compute the rank-1 update A = \alpha x y^T + A of the matrix A.*/
int fff_blas_dger (double alpha, const fff_vector * x, const fff_vector * y, fff_matrix * A)
{
  int incx = (int) x->stride; 
  int incy = (int) y->stride;
  int m = (int) A->size2; 
  int n = (int) A->size1; 
  int lda = (int) A->tda; 
 
  return( dger_(&m, &n, 
		&alpha, 
		y->data, &incy, 
		x->data, &incx, 
		A->data, &lda) ); 
}

/* 
Compute the symmetric rank-1 update A = \alpha x x^T + A of the
symmetric matrix A. Since the matrix A is symmetric only its upper
half or lower half need to be stored. When Uplo is CblasUpper then the
upper triangle and diagonal of A are used, and when Uplo is CblasLower
then the lower triangle and diagonal of A are used.
*/
int fff_blas_dsyr (CBLAS_UPLO_t Uplo, double alpha, const fff_vector * x, fff_matrix * A)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  int incx = (int) x->stride; 
  int n = (int) A->size1; 
  int lda = (int) A->tda; 

  return( dsyr_(uplo, &n, 
		&alpha, 
		x->data, &incx, 
		A->data, &lda ) ); 
}

/* 
These functions compute the symmetric rank-2 update A = \alpha x y^T +
\alpha y x^T + A of the symmetric matrix A. Since the matrix A is
symmetric only its upper half or lower half need to be stored. When
Uplo is CblasUpper then the upper triangle and diagonal of A are used,
and when Uplo is CblasLower then the lower triangle and diagonal of A
are used.
*/
int fff_blas_dsyr2 (CBLAS_UPLO_t Uplo, double alpha, 
		    const fff_vector * x, const fff_vector * y, fff_matrix * A)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  int incx = (int) x->stride; 
  int incy = (int) y->stride; 
  int n = (int) A->size1; 
  int lda = (int) A->tda; 

  return( dsyr2_(uplo, &n, 
		 &alpha, 
		 y->data, &incy, 
		 x->data, &incx, 
		 A->data, &lda) ); 
}



/****** BLAS 3 ******/ 

/*
Compute the matrix-matrix product and sum C = \alpha op(A) op(B) +
\beta C where op(A) = A, A^T, A^H for TransA = CblasNoTrans,
CblasTrans, CblasConjTrans and similarly for the parameter TransB.
*/
int fff_blas_dgemm (CBLAS_TRANSPOSE_t TransA, CBLAS_TRANSPOSE_t TransB, 
		    double alpha, const fff_matrix * A, const fff_matrix * B, double beta, fff_matrix * C)
{
  char* transa = FORTRAN_TRANS(TransA); 
  char* transb = FORTRAN_TRANS(TransB); 
  int m = C->size2; 
  int n = C->size1;
  int lda = (int) A->tda; 
  int ldb = (int) B->tda; 
  int ldc = (int) C->tda;
  int k = (TransB == CblasNoTrans) ? (int)B->size1 : (int)B->size2;

  return( dgemm_(transa, transb, &m, &n, &k, &alpha, 
		 B->data, &ldb, 
		 A->data, &lda, 
		 &beta, 
		 C->data, &ldc) ); 
}

/*
Compute the matrix-matrix product and sum C = \alpha A B + \beta C for
Side is CblasLeft and C = \alpha B A + \beta C for Side is CblasRight,
where the matrix A is symmetric. When Uplo is CblasUpper then the
upper triangle and diagonal of A are used, and when Uplo is CblasLower
then the lower triangle and diagonal of A are used.
*/
int fff_blas_dsymm (CBLAS_SIDE_t Side, CBLAS_UPLO_t Uplo, 
		    double alpha, const fff_matrix * A, const fff_matrix * B, double beta, fff_matrix * C)
{
  char* side = FORTRAN_SIDE(Side); 
  char* uplo = FORTRAN_UPLO(Uplo); 
  int m = C->size2; 
  int n = C->size1;
  int lda = (int) A->tda; 
  int ldb = (int) B->tda; 
  int ldc = (int) C->tda; 

  return ( dsymm_(side, uplo, &m, &n,
		  &alpha, 
		  A->data, &lda, 
		  B->data, &ldb,
		  &beta, 
		  C->data, &ldc) ); 
}

/*
Compute the matrix-matrix product B = \alpha op(A) B for Side is
CblasLeft and B = \alpha B op(A) for Side is CblasRight. The matrix A
is triangular and op(A) = A, A^T, A^H for TransA = CblasNoTrans,
CblasTrans, CblasConjTrans. When Uplo is CblasUpper then the upper
triangle of A is used, and when Uplo is CblasLower then the lower
triangle of A is used. If Diag is CblasNonUnit then the diagonal of A
is used, but if Diag is CblasUnit then the diagonal elements of the
matrix A are taken as unity and are not referenced.
*/
int fff_blas_dtrmm (CBLAS_SIDE_t Side, CBLAS_UPLO_t Uplo, CBLAS_TRANSPOSE_t TransA, CBLAS_DIAG_t Diag, 
		    double alpha, const fff_matrix * A, fff_matrix * B)
{
  char* side = FORTRAN_SIDE(Side); 
  char* uplo = FORTRAN_UPLO(Uplo); 
  char* transa = FORTRAN_TRANS(TransA); 
  char* diag = FORTRAN_DIAG(Diag); 
  int m = B->size2; 
  int n = B->size1;
  int lda = (int) A->tda; 
  int ldb = (int) B->tda; 

  
  return( dtrmm_(side, uplo, transa, diag, &m, &n,
		 &alpha, 
		 A->data, &lda, 
		 B->data, &ldb) ); 
  
}

/*
Compute the inverse-matrix matrix product B = \alpha op(inv(A))B for
Side is CblasLeft and B = \alpha B op(inv(A)) for Side is
CblasRight. The matrix A is triangular and op(A) = A, A^T, A^H for
TransA = CblasNoTrans, CblasTrans, CblasConjTrans. When Uplo is
CblasUpper then the upper triangle of A is used, and when Uplo is
CblasLower then the lower triangle of A is used. If Diag is
CblasNonUnit then the diagonal of A is used, but if Diag is CblasUnit
then the diagonal elements of the matrix A are taken as unity and are
not referenced.
*/
int fff_blas_dtrsm (CBLAS_SIDE_t Side, CBLAS_UPLO_t Uplo, CBLAS_TRANSPOSE_t TransA, CBLAS_DIAG_t Diag, 
		    double alpha, const fff_matrix * A, fff_matrix * B)
{
  char* side = FORTRAN_SIDE(Side); 
  char* uplo = FORTRAN_UPLO(Uplo); 
  char* transa = FORTRAN_TRANS(TransA); 
  char* diag = FORTRAN_DIAG(Diag); 
  int m = B->size2; 
  int n = B->size1;
  int lda = (int) A->tda; 
  int ldb = (int) B->tda; 

  return( dtrsm_(side, uplo, transa, diag, &m, &n, 
		 &alpha, 
		 A->data, &lda, 
		 B->data, &ldb) ); 
  
}

/*
Compute a rank-k update of the symmetric matrix C, C = \alpha A A^T +
\beta C when Trans is CblasNoTrans and C = \alpha A^T A + \beta C when
Trans is CblasTrans. Since the matrix C is symmetric only its upper
half or lower half need to be stored. When Uplo is CblasUpper then the
upper triangle and diagonal of C are used, and when Uplo is CblasLower
then the lower triangle and diagonal of C are used.
*/
int fff_blas_dsyrk (CBLAS_UPLO_t Uplo, CBLAS_TRANSPOSE_t Trans, 
		    double alpha, const fff_matrix * A, double beta, fff_matrix * C)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  char* trans = FORTRAN_TRANS(Trans); 
  int n = C->size1;
  int k = (Trans == CblasNoTrans) ? (int)A->size1 : (int)A->size2;
  int lda = (int) A->tda; 
  int ldc = (int) C->tda; 
  
  return( dsyrk_(uplo, trans, &n, &k,
		 &alpha, 
		 A->data, &lda, 
		 &beta,
		 C->data, &ldc) ); 
}

/* 
Compute a rank-2k update of the symmetric matrix C, C = \alpha A B^T +
\alpha B A^T + \beta C when Trans is CblasNoTrans and C = \alpha A^T B
+ \alpha B^T A + \beta C when Trans is CblasTrans. Since the matrix C
is symmetric only its upper half or lower half need to be stored. When
Uplo is CblasUpper then the upper triangle and diagonal of C are used,
and when Uplo is CblasLower then the lower triangle and diagonal of C
are used.
*/
int fff_blas_dsyr2k (CBLAS_UPLO_t Uplo, CBLAS_TRANSPOSE_t Trans, 
		     double alpha, const fff_matrix * A, const fff_matrix * B, double beta, fff_matrix * C)
{
  char* uplo = FORTRAN_UPLO(Uplo); 
  char* trans = FORTRAN_TRANS(Trans);
  int n = C->size1;
  int k = (Trans == CblasNoTrans) ? (int)B->size1 : (int)B->size2;
  int lda = (int) A->tda; 
  int ldb = (int) B->tda; 
  int ldc = (int) C->tda; 
 
  return( dsyr2k_(uplo, trans, &n, &k, 
		  &alpha, 
		  B->data, &ldb, 
		  A->data, &lda, 
		  &beta, 
		  C->data, &ldc) ); 
}
