#!/usr/bin/env python
"""Simple example of permutation testing.
"""

#-----------------------------------------------------------------------------
# Imports
#-----------------------------------------------------------------------------

# From the standard library
import os

# Third party imports
import numpy as np

# Imports from the fff project itself
from fff.misc import quantile
import fff.mfx as MFX
import fff.graph as G 

#-----------------------------------------------------------------------------
# Main code begins
#-----------------------------------------------------------------------------

# Get group data
data = np.load(os.path.join('data','offset_002.npz'))
MAT, VAR, XYZ = data.f.mat, data.f.var, data.f.xyz

# Compute Student statistical map and permute sample signs (exhaustively if
# nsubjects <= 10)
nsimu = min( 2**10, 2**MAT.shape[0] )
T = MFX.routines.test_stat( MAT, stat=MFX.routines.student, nsimu=nsimu )

# Compute single threshold to control the false positive rate at .01
Tth = quantile( T.ravel(), ratio = 1-1e-2 ) ## flatten to have a stationary
                                            ## threshold

# Compute the Tmax stat for each permutation 
Tmax = T.max(1) 

# Compute the Smax (max cluster size) stat for each permutation
# Use fff.graph module by B. Thirion  
Smax = np.zeros( nsimu )
for i in range(nsimu) :
    # Threshold map at the FPR controlling threshold 
    I = np.ravel(np.where( T[i]>=Tth ))
    XYZth = XYZ[:,I]

    if XYZth.shape[1] > 0 : 
        # Compute the graph associated with XYZth
        # Notice the coordinate matrix must be Npts x 3, so we need to transpose
        A, B, D = G.graph_3d_grid( XYZth.transpose() )   
        V = max(A) + 1 ## number of vertices 
        Smax[i] = np.size( G.graph_main_cc( A, B, D, V ) ) ## biggest cluster
                                                           ## size

print np.where(Smax>=Smax[0])
