#include "fffMat.h"
#include <string.h>


/* Input mxArray assumed DOUBLE
   Output fff_vector never owner */ 
fff_vector* fff_vector_from_mxArray( const mxArray *x )
{
  fff_vector* y;
  size_t nr = mxGetM( x );
  size_t nc = mxGetN( x );
  
  if ( FFF_MIN(nr, nc) != 1 ) {
    mexErrMsgTxt("Invalid input vector.");
    return NULL;
  }

  y = (fff_vector*) calloc( 1, sizeof(fff_vector) ); 
  y->size = (size_t) FFF_MAX(nr, nc); 
  y->stride = 1; 
  y->data = mxGetPr( x ); 
  y->owner = 0; 

  return y; 
}

/** list of fff_vectors <--> mxArray */ 
fff_vector** fff_vector_list_from_mxArray_new( const mxArray *x, int* nc )
{
  fff_vector** y; 
  int nr = mxGetM( x );
  int i;
  double* buf = mxGetPr( x ); 

  *nc = mxGetN( x );

  y = (fff_vector**) malloc( (*nc) * sizeof(fff_vector*) ); 

  for (i=0; i<(*nc); i++, buf+=nr) {

    y[i] = (fff_vector*)malloc( sizeof(fff_vector) );
    y[i]->size = (size_t) nr;
    y[i]->stride = 1;  
    y[i]->data = buf; 
    y[i]->owner = 0; 

  }

  return y; 
}

void fff_vector_list_from_mxArray_delete( fff_vector **x, int nc )
{
  int i; 

  for( i=0; i<nc; i++ ) 
    free( x[i] ); 
  free( x ); 

  return; 
}



int mxArray_is_long( const mxArray* x, mxClassID* mxLong )
{
  int ok = 1; 
  int sizeLong = sizeof(long); /* should be 4 (resp. 8) on a 32-bit (resp. 64-bit) system */
  
  /* Default */ 
  *mxLong = mxINT32_CLASS; 

  if ( sizeLong == 2 ) { /* Very unlikely ! */ 
    *mxLong = mxINT16_CLASS; 
    if ( ! mxIsInt16( x ) )
      ok = 0; 
  }
  if ( sizeLong == 4 ) {
    if ( ! mxIsInt32( x ) )
      ok = 0; 
  }
  else if ( sizeLong == 8 ) {
    *mxLong = mxINT64_CLASS; 
    if ( ! mxIsInt64( x ) )
      ok = 0; 
  }
  else {
    ok = 0; 
    mexErrMsgTxt("Unrecognized long int format.");
  }

  return ok; 
} 



fff_array* fff_array_from_mxArray( const mxArray *ptr ) {
  
  fff_array *x; 
  const int *dim = mxGetDimensions(ptr);
  mxClassID mx_datatype = mxGetClassID(ptr); 
  fff_datatype datatype; 
  unsigned int dimX, dimY, dimZ, dimT; 
  unsigned int offX, offY, offZ, offT; 
  unsigned int ndims, aux = 1; 
  
  ndims = mxGetNumberOfDimensions( ptr );
  if ( ( ndims<2 ) || ( ndims>4 ) ) {
    mexErrMsgTxt("Impossible conversion: input array has too few or too many dimensions.");
    return NULL;
  }

  switch ( mx_datatype ) {
  case mxUINT8_CLASS: 
    datatype = fff_get_datatype( 1, 1, 0 );
    break;
  case mxINT8_CLASS: 
    datatype = fff_get_datatype( 1, 1, 1 );
    break;
  case mxUINT16_CLASS: 
    datatype = fff_get_datatype( 2, 1, 0 );
    break;
  case mxINT16_CLASS: 
    datatype = fff_get_datatype( 2, 1, 1 );
    break;
  case mxUINT32_CLASS: 
    datatype = fff_get_datatype( 4, 1, 0 );
    break;
  case mxINT32_CLASS: 
    datatype = fff_get_datatype( 4, 1, 1 );
    break;
  case mxUINT64_CLASS: 
    datatype = fff_get_datatype( 8, 1, 0 );
    break;
  case mxINT64_CLASS: 
    datatype = fff_get_datatype( 8, 1, 1 );
    break;
  case mxSINGLE_CLASS: 
    datatype = fff_get_datatype( 4, 0, 1 );
    break;
  case mxDOUBLE_CLASS: 
    datatype = fff_get_datatype( 8, 0, 1 );
    break;
  default: 
    datatype = FFF_UNKNOWN_TYPE; 
    break; 
  }

  /* Is that a known data type ? */ 
  if ( datatype == FFF_UNKNOWN_TYPE ) {
    mexErrMsgTxt("Unrecognized data type.");
    return NULL;
  }

  /* Dimensions and offsets */ 
  dimX = (unsigned int)(dim[0]);
  offX = aux;
  aux *= dimX;   
  dimY = (unsigned int)(dim[1]); 
  offY = aux; 
  aux *= dimY; 
  if ( ndims > 2 ) {
    dimZ = (unsigned int)(dim[2]); 
    offZ = aux; 
    aux *= dimZ; 
    if ( ndims > 3 ) {
      dimT = (unsigned int)(dim[3]); 
      offT = aux; 
    }
    else {
      dimT = 1; 
      offT = aux; 
    }
  } 
  else {
    dimZ = 1; 
    offZ = aux; 
    dimT = 1; 
    offT = aux; 
  }
    

  /* Create structure */ 
  x = fff_array_from_data_new( datatype, 
			       (void*) mxGetPr(ptr), 
			       dimX, dimY, dimZ, dimT, 
			       offX, offY, offZ, offT ); 


  return x; 
 
}


