#include "fffMat.h"

#include <fff_routines.h>

/* 
   
Y = fffMat_quantile( X, P )

X : two-dimensional array 
P : value between 0 and 1 


*/ 

void mexFunction( int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray*prhs[] )
{ 
  /* Variables */ 
  gsl_vector **X; 
  gsl_vector *Y; 
  gsl_vector *aux; 
  double P; 
  int i, nvox, flagInterp = 0; 

  /* Check for proper number of arguments. */
  if ( nrhs < 2 ) {
    mexErrMsgTxt("Two input arguments required.");
  } 
  else if ( nlhs > 1 ) 
    mexErrMsgTxt("Too many output arguments");


  /* Parse input */
  X = gsl_vector_list_from_mxArray_new( prhs[0], &nvox );
  P = *mxGetPr( prhs[1] );
  if ( nrhs > 2 ) 
    flagInterp = (int) ( *mxGetPr( prhs[2] ) ); 

  /* Allocate output */ 
  plhs[0] = mxCreateDoubleMatrix( 1, nvox, mxREAL );
  Y = gsl_vector_from_mxArray( plhs[0] );
  
  /* Local variables */ 
  aux = gsl_vector_alloc( X[0]->size ); 
  
  /* Loop over voxels */ 
  for( i=0; i<nvox; i++ ) {
    gsl_vector_memcpy( aux, X[i] ); 
    gsl_vector_set ( Y, i, fff_quantile_from_temp_data( aux, P, flagInterp ) ); 
  }


  /* Free memory */ 
  gsl_vector_list_from_mxArray_delete( X, nvox );
  gsl_vector_free( Y ); 
  gsl_vector_free( aux ); 

  return; 
}

