static PyObject* call_blas1(PyObject* self, PyObject* args)
{
	PyArrayObject *py_x, *py_y, *py_res;
	fff_vector *x, *y, *res;
	double alpha; 

	/* Check that input object is an array */
	if (!PyArg_ParseTuple(args, "O!O!d:call_blas1", 
			      &PyArray_Type, &py_x, 
			      &PyArray_Type, &py_y, 
			      &alpha))
		Py_RETURN_NONE;

	x = fff_vector_fromPyArray( py_x );
	y = fff_vector_fromPyArray( py_y );
	
	res = fff_vector_new(8); 

	/* Start with basic operations */ 
	fff_vector_add(x, y); 
	fff_vector_mul(x, y);
	fff_vector_sub(y, x); 
	fff_vector_div(y, x); 

	/* Test 1: ddot */ 
	fprintf(stderr, "Testing BLAS1: ddot\n"); 
	res->data[0] = fff_blas_ddot(x, y); 

	/* Test 2: dnrm2 */ 
	fprintf(stderr, "Testing BLAS1: dnrm2\n"); 
	res->data[1] = fff_blas_dnrm2(x); 

	/* Test 3: dasum */ 
	fprintf(stderr, "Testing BLAS1: dasum\n"); 
	res->data[2] = fff_blas_dasum(x); 

	/* Test 4: idamax */ 
	fprintf(stderr, "Testing BLAS1: idamax\n"); 
	res->data[3] = (double)fff_blas_idamax(x); 

	/* Test 5: daxpy */
	fprintf(stderr, "Testing BLAS1: daxpy\n"); 
	fff_blas_daxpy(alpha, x, y); 
	res->data[4] = fff_blas_dnrm2(y); 

	/* Test 6: dscal */
	fprintf(stderr, "Testing BLAS1: dscal\n"); 
	fff_blas_dscal(alpha, x);  
	res->data[5] = fff_blas_dasum(x); 
	
	/* Test 7: dswap */ 
	fprintf(stderr, "Testing BLAS1: dswap\n"); 
	fff_blas_dswap(x, y); 
	res->data[6] = fff_blas_dasum(x);

	/* Test 8: dcopy */ 
	fprintf(stderr, "Testing BLAS1: dcopy\n"); 
	fff_blas_dcopy(x, y); 
	fff_vector_sub(x, y); 
	res->data[7] = fff_blas_dnrm2(x); 

	py_res = fff_vector_toPyArray( res );

	return PyArray_Return( py_res );
}
