static PyObject* call_blas2(PyObject* self, PyObject* args)
{
	PyArrayObject *py_A, *py_x, *py_y, *py_res;
	fff_matrix* A; 
	fff_vector *x, *y, *res;
	double alpha, beta; 

	/* Check that input object is an array */
	if (!PyArg_ParseTuple(args, "O!O!O!dd:call_blas2", 
			      &PyArray_Type, &py_A, 
			      &PyArray_Type, &py_x, 
			      &PyArray_Type, &py_y, 
			      &alpha, &beta))
		Py_RETURN_NONE;

	A = fff_matrix_fromPyArray( py_A );
	x = fff_vector_fromPyArray( py_x );
	y = fff_vector_fromPyArray( py_y );
	
	res = fff_vector_new(12); 

	/* Tests: dgemv */ 
	fprintf(stderr, "Testing BLAS2: dgemv\n"); 
	fff_blas_dgemv( CblasNoTrans, alpha, A, x, beta, y );  
	res->data[0] = fff_blas_dasum(y);
	fff_blas_dgemv( CblasTrans, alpha, A, x, beta, y );  
	res->data[1] = fff_blas_dasum(y);

	/* Tests: dtrmv */ 
	fprintf(stderr, "Testing BLAS2: dtrmv\n"); 
	fff_blas_dtrmv(CblasUpper, CblasNoTrans, CblasNonUnit, A, x); 
	res->data[2] = fff_blas_dasum(x);
	fff_blas_dtrmv(CblasUpper, CblasTrans, CblasNonUnit, A, x); 
	res->data[3] = fff_blas_dasum(x);
	fff_blas_dtrmv(CblasLower, CblasNoTrans, CblasNonUnit, A, y); 
	res->data[4] = fff_blas_dasum(y);
	fff_blas_dtrmv(CblasLower, CblasTrans, CblasUnit, A, y); 
	res->data[5] = fff_blas_dasum(y);
	
	/* Tests: dtrsv */ 
	fprintf(stderr, "Testing BLAS2: dtrsv\n"); 
	fff_blas_dtrsv(CblasUpper, CblasNoTrans, CblasNonUnit, A, x); 
	res->data[6] = fff_blas_dasum(x);
	fff_blas_dtrsv(CblasUpper, CblasTrans, CblasNonUnit, A, x); 
	res->data[7] = fff_blas_dasum(x);
	fff_blas_dtrsv(CblasLower, CblasTrans, CblasNonUnit, A, y); 
	res->data[8] = fff_blas_dasum(y);
	fff_blas_dtrsv(CblasLower, CblasNoTrans, CblasUnit, A, y); 
	res->data[9] = fff_blas_dasum(y);

	/* Tests: */ 
	fprintf(stderr, "Testing BLAS2: dsymv\n"); 
	fff_blas_dsymv (CblasUpper, alpha, A, x, beta, y); 
	res->data[10] = fff_blas_dasum(y);
	fff_blas_dsymv (CblasLower, alpha, A, x, beta, y); 
	res->data[11] = fff_blas_dasum(y);

	py_res = fff_vector_toPyArray( res );

	return PyArray_Return( py_res );
}
