/*!
  \file fff_macro.h
  \brief General macros for fff
  \author Alexis Roche
  \date 2003-2006

*/

#ifndef fff_MACRO
#define fff_MACRO
 
#ifdef __cplusplus
extern "C" {
#endif


  /* Global FFF compilation options (to be moved somewhere else) */ 
#define fff_VERBOSE 1

  /* Some useful macros (use GSL macros when they exist!) */
  /*!
    Rounds \a a to the nearest smaller integer 
    \bug Compilator-dependent?
  */
#define fff_FLOOR(a)((a)>0.0 ? (int)(a):(((int)(a)-a)!= 0.0 ? (int)(a)-1 : (int)(a)))  
  /*!
    Rounds \a a to the nearest integer (either smaller or bigger) 
  */
#define fff_ROUND(a)(fff_FLOOR(a+0.5))
  /*!
    Rounds \a a to the nearest bigger integer 
  */
#define fff_CEIL(a)(-(fff_FLOOR(-(a))))
  /*!
    Rounds \a a to the nearest smaller integer, assuming \a a is non-negative 
    \bug Compilator-dependent?
  */
#define fff_UNSIGNED_FLOOR(a) ( (int)(a) )
  /*!
    Rounds \a a to the nearest integer, assuming \a a is non-negative 
  */
#define fff_UNSIGNED_ROUND(a) ( (int)(a+0.5) )
  /*!
    Rounds \a a to the nearest bigger integer, assuming \a a is non-negative 
  */
#define fff_UNSIGNED_CEIL(a) ( ( (int)(a)-a )!=0.0 ? (int)(a+1) : (int)(a) )
  /*!
    Returns 1 if \a a is positive, -1 if \a a is negative, 0 if \a a equals zero

    Note that this macro differs from \a GSL_SIGN which returns +1 if \a a==0
  */
#define fff_SIGN(a)( (a)>0.0 ? 1 : ( (a)<0.0 ? -1 : 0 ) )
  /*!
    Computes the absolute value of \a a
  */
#define fff_ABS(a) ( (a) > 0.0 ? (a) : (-(a)) )
  /*!
    Computes \f$ a^2 \f$
  */
#define fff_SQR(a) ( (a)*(a) )
  /*!
    Computes \f$ a^3 \f$
  */
#define fff_CUBE(a) ( (a)*(a)*(a) )
  /*!
    Computes \f$ a modulo, b ie the remainder after division of a by b  \f$
  */
#define fff_REM(a, b) ( (int)(a)%(int)(b) )
  /*!
    Computes the minimum of \a a and \a b
  */
#define fff_MIN(a,b) ( (a) < (b) ? (a) : (b) ) 
  /*!
    Low threshold a value to avoid vanishing 
  */
#define fff_TINY 1e-50
#define fff_ENSURE_POSITIVE(a) ( (a) > fff_TINY ? (a) : fff_TINY )

#ifdef __cplusplus
}
#endif
 
#endif  
