#!/usr/bin/env python
"""Little script to generate a summary of all the GSL dependencies in fff.

It reads a 'raw' file generated from a brute-force grep of the entire fff
original source tree and produces a sorted summary of all names with 'gsl' in
them.
"""

import re

def hist2str(hist,mode='key',reverse=True):
    """Nicely format a histogram stored as a dict with strings as keys.
    """
    assert mode in ('key','value'),"Mode must be one of 'key','value'"
    key = None if mode=='key' else hist.get
    
    # ensure the dict has string keys
    shist = dict((str(k),hist[k]) for k in hist)
    # Make a generator expression to iterates over the dict in the needed order
    slist = list((k,shist[k]) for k in sorted(shist,key=key,reverse=reverse))
    # Find widest key as a string
    kwidth = max(len(s[0]) for s in slist)
    # Generate output
    out = []
    tpl = '%-' + str(kwidth) + 's ---> %s'
    return '\n'.join([tpl % kv  for kv in slist ])

if __name__ == '__main__':
    MODE = 'key'  # sort output by function names
    MODE = 'value'  # sort output by call counts

    REVERSE = True if MODE=='value' else False
    
    raw = open('deps_gsl_raw.txt')

    gsl = {}  # original gsl deps
    fff = {}  # gsl names in fff-written code (mainly to interface *with* gsl)

    for l in raw:
        match = re.search('.*?(_?gsl_\w*)\(',l)
        if match:
            mg = match.group(1).strip()
            if 'Array' in mg or 'Ptr' in mg:
                tab = fff
            else:
                tab = gsl

            tab.setdefault(mg,1)
            tab[mg] += 1

    # All fff numbers are off by two, because we parsed the header declaration
    # and the function implementation.  We only want actual call numbers
    for k in fff:
        fff[k] -= 2

    # Summary info
    print 'GSL dependencies'
    print '================'
    print hist2str(gsl,MODE,REVERSE)
    print '\n'
    print 'In-house GSL-related routines'
    print '============================='
    print hist2str(fff,MODE,REVERSE)
