/*!
  \file fff_gen_stats.h
  \brief General interest statistical routines
  \author Alexis Roche
  \date 2004-2008

*/


 
#ifndef FFF_GEN_STATS
#define FFF_GEN_STATS

#ifdef __cplusplus
extern "C" {
#endif
  
#include "fff_vector.h"
#include "fff_matrix.h"

  
  /*!
    \brief Squared Mahalanobis distance
    \param x input data vector (beware: gets modified)
    \param S associated variance matrix
    \param Saux auxiliary matrix, same size as \a S
    
    Compute the squared Mahalanobis distance \f$ d^2 = x^t S^{-1} x
    \f$. The routine uses the Cholesky decomposition: \f$ S = L L^t
    \f$ where \a L is lower triangular, and then exploits the fact
    that \f$ d^2 = \| L^{-1}x \|^2 \f$.
  */  
  extern double fff_mahalanobis( fff_vector* x, fff_matrix* S, fff_matrix* Saux );

  /*
	\brief Draw integers without replacement
	\param x the resulting integer list
	\param n the number of integers to be drawn (the list length)
	\param N the upper bound defining the set of integers 

   */
  extern void fff_rng_draw_noreplace(size_t* x, unsigned int n, unsigned int N);

#ifdef __cplusplus
}
#endif

#endif

